require 'notmuch'
require 'mail'
require 'rack/mime'

$db_name = nil
$all_emails = []
$email = $email_name = $email_address = nil
$exclude_tags = []
$searches = []

def show(thread_id, msg_id)
  showheaders = ["Subject", "To", "Cc", "Date", "Message-Id"]

  $curbuf.cur_thread = thread_id
  messages = $curbuf.messages
  buffer = $curbuf
    q = $curbuf.query(get_cur_view)
    q.sort = Notmuch::SORT_OLDEST_FIRST
    msgs = q.search_messages
    msgs.each do |msg|
      m = Mail.read(msg.filename)
      enc = false
      mime = false
      encfail = false
      text_part = m.find_first_text
      nm_m = Message.new(msg, m)
      messages << nm_m
      date = msg.date
      nm_m.start = buffer.count
      buffer << 'From: %s %s (%s)' % [msg['from'], date, msg.tags]
      showheaders.each do |h|
        buffer << '%s: %s' % [h, m.header[h]]
      end
      nm_m.full_header_start = buffer.count
      nm_m.body_start = buffer.count
      text_part.convert.each_line do |l|
        buffer << l.chomp
      end
      buffer << ''
      nm_m.end = buffer.count
    end
  messages = $curbuf.messages
  messages.each_with_index do |msg, i|
    if msg.tags.include?('unread')
      puts "normal #{msg.start}G"
      puts 'foldopen'
    end
  end
end

def get_cur_view
  if $cur_filter
    return "#{$curbuf.cur_thread} and (#{$cur_filter})"
  else
    return $curbuf.cur_thread
  end
end

module DbHelper
  def init_dbhelper
    @db = Notmuch::Database.new($db_name)
    @queries = []
  end

  def query(*args)
    q = @db.query(*args)
    @queries << q
    q
  end

  def message(id)
    @db.find_message(id)
  end
end

module URI
  class MsgID < Generic
  end

  @@schemes['ID'] = MsgID
end

class Message
  attr_accessor :start, :body_start, :end, :full_header_start, :full_header_end
  attr_reader :message_id, :filename, :mail, :tags

  def initialize(msg, mail)
    @message_id = msg.message_id
    @filename = msg.filename
    @mail = mail
    @start = 0
    @end = 0
    @full_header_start = 0
    @full_header_end = 0
    @tags = msg.tags
  end

  def to_s
    'id:%s' % @message_id
  end

  def inspect
    'id:%s, file:%s' % [@message_id, @filename]
  end
end

class Buffer
  include DbHelper
  attr_accessor :messages, :threads, :cur_thread, :lines

  def initialize(*args)
    super
    @messages = []
    @threads = []
    @lines = []

    init_dbhelper()
  end

  def line_count
    @lines.count
  end

  def <<(message)
    message.split("\n").each {
      |s| append(count(), s)
    }
  end

  def count
    @lines.count
  end

  def append(index, str)
    @lines << str
    str
  end
end

class Notmuch::Tags
  def to_s
    to_a.join(' ')
  end
end

class Notmuch::Message
  def to_s
    'id:%s' % message_id
  end
end

module Mail
  class Message
    def find_first_text
      return self if not multipart?
      return text_part || html_part || self
    end

    def convert
      if mime_type != 'text/html'
        begin
          text = decoded
        rescue Exception
          text = decode_body
        end
      end
      text
    end
  end
end

def get_config_item(item)
  result = ''
  IO.popen(['notmuch', 'config', 'get', item]) { |out|
    result = out.read
  }
  return result.rstrip
end

def get_config
  $db_name = get_config_item('database.path')
  $email_name = get_config_item('user.name')
  $email_address = get_config_item('user.primary_email')
  $secondary_email_addresses = get_config_item('user.primary_email')
  $email = '%s <%s>' % [$email_name, $email_address]
  other_emails = get_config_item('user.other_email')
  $all_emails = other_emails.split("\n")
  # Add the primary to this too as we use it for checking
  # addresses when doing a reply
  $all_emails.unshift($email_address)
  ignore_tags = get_config_item('search.exclude_tags')
  $exclude_tags = ignore_tags.split("\n")
end

get_config

$curbuf = Buffer.new()
show("thread:0000000000000001", "")
#puts $curbuf.lines
